@@ -23,8 +23,7 @@ from email.parser import Parser
 import io
 import json
 
-import requests
-import six
+import requests  # type: ignore
 
 from google.cloud import _helpers
 from google.cloud import exceptions
@@ -65,13 +64,7 @@ class MIMEApplicationHTTP(MIMEApplication):
         lines.append("")
         lines.append(body)
         payload = "\r\n".join(lines)
-        if six.PY2:
-            # email.message.Message is an old-style class, so we
-            # cannot use 'super()'.
-            MIMEApplication.__init__(self, payload, "http", encode_noop)
-        else:  # pragma: NO COVER  Python3
-            super_init = super(MIMEApplicationHTTP, self).__init__
-            super_init(payload, "http", encode_noop)
+        super().__init__(payload, "http", encode_noop)
 
 
 class _FutureDict(object):
@@ -219,11 +212,7 @@ class Batch(Connection):
             multi.attach(subrequest)
             timeout = _timeout
 
-        # The `email` package expects to deal with "native" strings
-        if six.PY2:  # pragma: NO COVER  Python3
-            buf = io.BytesIO()
-        else:
-            buf = io.StringIO()
+        buf = io.StringIO()
         generator = Generator(buf, False, 0)
         generator.flatten(multi)
         payload = buf.getvalue()
@@ -315,10 +304,7 @@ def _generate_faux_mime_message(parser, response):
         [b"Content-Type: ", content_type, b"\nMIME-Version: 1.0\n\n", response.content]
     )
 
-    if six.PY2:
-        return parser.parsestr(faux_message)
-    else:  # pragma: NO COVER  Python3
-        return parser.parsestr(faux_message.decode("utf-8"))
+    return parser.parsestr(faux_message.decode("utf-8"))
 
 
 def _unpack_batch_response(response):
