@@ -25,14 +25,16 @@ import google.api_core.client_options
 
 from google.auth.credentials import AnonymousCredentials
 
-from google import resumable_media
+from google import resumable_media  # type: ignore
 
 from google.api_core import page_iterator
 from google.cloud._helpers import _LocalStack, _NOW
 from google.cloud.client import ClientWithProject
 from google.cloud.exceptions import NotFound
+from google.cloud.storage._helpers import _get_default_headers
 from google.cloud.storage._helpers import _get_environ_project
 from google.cloud.storage._helpers import _get_storage_host
+from google.cloud.storage._helpers import _BASE_STORAGE_URI
 from google.cloud.storage._helpers import _DEFAULT_STORAGE_HOST
 from google.cloud.storage._helpers import _bucket_bound_hostname_url
 from google.cloud.storage._helpers import _add_etag_match_headers
@@ -146,7 +148,7 @@ class Client(ClientWithProject):
         # STORAGE_EMULATOR_HOST or a non-default api_endpoint is set.
         if (
             kw_args["api_endpoint"] is not None
-            and kw_args["api_endpoint"].find("storage.googleapis.com") < 0
+            and _BASE_STORAGE_URI not in kw_args["api_endpoint"]
         ):
             if credentials is None:
                 credentials = AnonymousCredentials()
@@ -932,12 +934,22 @@ class Client(ClientWithProject):
 
         """
         bucket = self._bucket_arg_to_bucket(bucket_or_name)
+        query_params = {}
 
         if project is None:
             project = self.project
 
-        if project is None:
-            raise ValueError("Client project not set:  pass an explicit project.")
+        # Use no project if STORAGE_EMULATOR_HOST is set
+        if _BASE_STORAGE_URI not in _get_storage_host():
+            if project is None:
+                project = _get_environ_project()
+            if project is None:
+                project = "<none>"
+
+        # Only include the project parameter if a project is set.
+        # If a project is not set, falls back to API validation (BadRequest).
+        if project is not None:
+            query_params = {"project": project}
 
         if requester_pays is not None:
             warnings.warn(
@@ -947,8 +959,6 @@ class Client(ClientWithProject):
             )
             bucket.requester_pays = requester_pays
 
-        query_params = {"project": project}
-
         if predefined_acl is not None:
             predefined_acl = BucketACL.validate_predefined(predefined_acl)
             query_params["predefinedAcl"] = predefined_acl
@@ -1089,7 +1099,7 @@ class Client(ClientWithProject):
             >>> from google.cloud import storage
             >>> client = storage.Client()
 
-            >>> with open('file-to-download-to', 'w') as file_obj:
+            >>> with open('file-to-download-to', 'wb') as file_obj:
             >>>     client.download_blob_to_file(
             >>>         'gs://bucket_name/path/to/blob', file_obj)
 
@@ -1122,6 +1132,7 @@ class Client(ClientWithProject):
         _add_etag_match_headers(
             headers, if_etag_match=if_etag_match, if_etag_not_match=if_etag_not_match,
         )
+        headers = {**_get_default_headers(self._connection.user_agent), **headers}
 
         transport = self._http
         try:
@@ -1375,13 +1386,22 @@ class Client(ClientWithProject):
         :returns: Iterator of all :class:`~google.cloud.storage.bucket.Bucket`
                   belonging to this project.
         """
+        extra_params = {}
+
         if project is None:
             project = self.project
 
-        if project is None:
-            raise ValueError("Client project not set:  pass an explicit project.")
+        # Use no project if STORAGE_EMULATOR_HOST is set
+        if _BASE_STORAGE_URI not in _get_storage_host():
+            if project is None:
+                project = _get_environ_project()
+            if project is None:
+                project = "<none>"
 
-        extra_params = {"project": project}
+        # Only include the project parameter if a project is set.
+        # If a project is not set, falls back to API validation (BadRequest).
+        if project is not None:
+            extra_params = {"project": project}
 
         if prefix is not None:
             extra_params["prefix"] = prefix
