@@ -24,12 +24,10 @@ import uuid
 
 import pytest
 import requests
-
-from six.moves.urllib import parse as urlparse
+import urllib
 
 from google.auth.credentials import AnonymousCredentials
 from google.cloud import storage
-from google.cloud.exceptions import NotFound
 from google.cloud.storage.hmac_key import HMACKeyMetadata
 
 from . import _read_local_json
@@ -39,7 +37,7 @@ _CONFORMANCE_TESTS = _read_local_json("retry_strategy_test_data.json")["retryTes
 
 """Environment variable or default host for Storage testbench emulator."""
 _HOST = os.environ.get("STORAGE_EMULATOR_HOST", "http://localhost:9000")
-_PORT = urlparse.urlsplit(_HOST).port
+_PORT = urllib.parse.urlsplit(_HOST).port
 
 """The storage testbench docker image info and commands."""
 _DEFAULT_IMAGE_NAME = "gcr.io/cloud-devrel-public-resources/storage-testbench"
@@ -56,6 +54,7 @@ _CONF_TEST_PUBSUB_TOPIC_NAME = "my-topic-name"
 
 _STRING_CONTENT = "hello world"
 _BYTE_CONTENT = b"12345678"
+_RESUMABLE_UPLOAD_CHUNK_SIZE = 2 * 1024 * 1024
 
 
 ########################################################################################################################################
@@ -79,39 +78,72 @@ def blob_exists(client, _preconditions, **resources):
 
 def blob_download_as_bytes(client, _preconditions, **resources):
     bucket = resources.get("bucket")
-    object = resources.get("object")
-    blob = client.bucket(bucket.name).blob(object.name)
-    blob.download_as_bytes()
+    file, data = resources.get("file_data")
+    # download the file and assert data integrity
+    blob = client.bucket(bucket.name).blob(file.name)
+    stored_contents = blob.download_as_bytes()
+    assert stored_contents == data.encode("utf-8")
+
+
+def blob_download_as_bytes_w_range(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    file, data = resources.get("file_data")
+    blob = client.bucket(bucket.name).blob(file.name)
+    start_byte = 0
+    end_byte = 1000000
+    stored_contents = blob.download_as_bytes(start=start_byte, end=end_byte - 1)
+    assert stored_contents == data.encode("utf-8")[start_byte:end_byte]
 
 
 def blob_download_as_text(client, _preconditions, **resources):
     bucket = resources.get("bucket")
-    object = resources.get("object")
-    blob = client.bucket(bucket.name).blob(object.name)
-    blob.download_as_text()
+    file, data = resources.get("file_data")
+    blob = client.bucket(bucket.name).blob(file.name)
+    stored_contents = blob.download_as_text()
+    assert stored_contents == data
 
 
 def blob_download_to_filename(client, _preconditions, **resources):
     bucket = resources.get("bucket")
-    object = resources.get("object")
-    blob = client.bucket(bucket.name).blob(object.name)
+    file, data = resources.get("file_data")
+    blob = client.bucket(bucket.name).blob(file.name)
     with tempfile.NamedTemporaryFile() as temp_f:
         blob.download_to_filename(temp_f.name)
+        with open(temp_f.name, "r") as file_obj:
+            stored_contents = file_obj.read()
+    assert stored_contents == data
+
+
+def blob_download_to_filename_chunked(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    file, data = resources.get("file_data")
+    blob = client.bucket(bucket.name).blob(file.name, chunk_size=40 * 1024 * 1024)
+    with tempfile.NamedTemporaryFile() as temp_f:
+        blob.download_to_filename(temp_f.name)
+        with open(temp_f.name, "r") as file_obj:
+            stored_contents = file_obj.read()
+    assert stored_contents == data
 
 
 def client_download_blob_to_file(client, _preconditions, **resources):
-    object = resources.get("object")
+    bucket = resources.get("bucket")
+    file, data = resources.get("file_data")
+    blob = client.bucket(bucket.name).blob(file.name)
     with tempfile.NamedTemporaryFile() as temp_f:
         with open(temp_f.name, "wb") as file_obj:
-            client.download_blob_to_file(object, file_obj)
+            client.download_blob_to_file(blob, file_obj)
+        with open(temp_f.name, "r") as to_read:
+            stored_contents = to_read.read()
+    assert stored_contents == data
 
 
 def blobreader_read(client, _preconditions, **resources):
     bucket = resources.get("bucket")
-    object = resources.get("object")
-    blob = client.bucket(bucket.name).blob(object.name)
-    with blob.open() as reader:
-        reader.read()
+    file, data = resources.get("file_data")
+    blob = client.bucket(bucket.name).blob(file.name)
+    with blob.open(mode="r") as reader:
+        stored_contents = reader.read()
+    assert stored_contents == data
 
 
 def client_list_blobs(client, _preconditions, **resources):
@@ -428,35 +460,79 @@ def blob_compose(client, _preconditions, **resources):
 
 def blob_upload_from_string(client, _preconditions, **resources):
     bucket = resources.get("bucket")
+    _, data = resources.get("file_data")
     blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
+    blob.chunk_size = _RESUMABLE_UPLOAD_CHUNK_SIZE
     if _preconditions:
-        blob.upload_from_string(_STRING_CONTENT, if_generation_match=0)
+        blob.upload_from_string(data, if_generation_match=0)
     else:
-        blob.upload_from_string(_STRING_CONTENT)
+        blob.upload_from_string(data)
+    assert blob.size == len(data)
 
 
 def blob_upload_from_file(client, _preconditions, **resources):
     bucket = resources.get("bucket")
-    blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
+    file, data = resources.get("file_data")
+    file_blob = client.bucket(bucket.name).blob(file.name)
+    upload_blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
+    upload_blob.chunk_size = _RESUMABLE_UPLOAD_CHUNK_SIZE
+
     with tempfile.NamedTemporaryFile() as temp_f:
+        # Create a named temporary file with payload.
+        with open(temp_f.name, "wb") as file_obj:
+            client.download_blob_to_file(file_blob, file_obj)
+        # Upload the temporary file and assert data integrity.
         if _preconditions:
-            blob.upload_from_file(temp_f, if_generation_match=0)
+            upload_blob.upload_from_file(temp_f, if_generation_match=0)
         else:
-            blob.upload_from_file(temp_f)
+            upload_blob.upload_from_file(temp_f)
+
+    upload_blob.reload()
+    assert upload_blob.size == len(data)
 
 
 def blob_upload_from_filename(client, _preconditions, **resources):
     bucket = resources.get("bucket")
     blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
+    blob.chunk_size = _RESUMABLE_UPLOAD_CHUNK_SIZE
+
+    bucket = resources.get("bucket")
+    file, data = resources.get("file_data")
+    file_blob = client.bucket(bucket.name).blob(file.name)
+    upload_blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
+    upload_blob.chunk_size = _RESUMABLE_UPLOAD_CHUNK_SIZE
 
     with tempfile.NamedTemporaryFile() as temp_f:
+        # Create a named temporary file with payload.
+        with open(temp_f.name, "wb") as file_obj:
+            client.download_blob_to_file(file_blob, file_obj)
+        # Upload the temporary file and assert data integrity.
         if _preconditions:
-            blob.upload_from_filename(temp_f.name, if_generation_match=0)
+            upload_blob.upload_from_filename(temp_f.name, if_generation_match=0)
         else:
-            blob.upload_from_filename(temp_f.name)
+            upload_blob.upload_from_filename(temp_f.name)
+
+    upload_blob.reload()
+    assert upload_blob.size == len(data)
 
 
 def blobwriter_write(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    _, data = resources.get("file_data")
+    blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
+    if _preconditions:
+        with blob.open(
+            "w", chunk_size=_RESUMABLE_UPLOAD_CHUNK_SIZE, if_generation_match=0
+        ) as writer:
+            writer.write(data)
+    else:
+        with blob.open("w", chunk_size=_RESUMABLE_UPLOAD_CHUNK_SIZE) as writer:
+            writer.write(data)
+    blob.reload()
+    assert blob.size == len(data)
+
+
+def blobwriter_write_multipart(client, _preconditions, **resources):
     chunk_size = 256 * 1024
     bucket = resources.get("bucket")
     blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
@@ -468,6 +544,15 @@ def blobwriter_write(client, _preconditions, **resources):
             writer.write(_BYTE_CONTENT)
 
 
+def blob_upload_from_string_multipart(client, _preconditions, **resources):
+    bucket = resources.get("bucket")
+    blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
+    if _preconditions:
+        blob.upload_from_string(_STRING_CONTENT, if_generation_match=0)
+    else:
+        blob.upload_from_string(_STRING_CONTENT)
+
+
 def blob_create_resumable_upload_session(client, _preconditions, **resources):
     bucket = resources.get("bucket")
     blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
@@ -672,10 +757,20 @@ method_mapping = {
         blob_exists,
         client_download_blob_to_file,
         blob_download_to_filename,
+        blob_download_to_filename_chunked,
         blob_download_as_bytes,
         blob_download_as_text,
         blobreader_read,
     ],
+    "storage.objects.download": [
+        client_download_blob_to_file,
+        blob_download_to_filename,
+        blob_download_to_filename_chunked,
+        blob_download_as_bytes,
+        blob_download_as_bytes_w_range,
+        blob_download_as_text,
+        blobreader_read,
+    ],
     "storage.objects.list": [client_list_blobs, bucket_list_blobs, bucket_delete],
     "storage.serviceaccount.get": [client_get_service_account_email],  # S1 end
     "storage.buckets.patch": [
@@ -701,11 +796,15 @@ method_mapping = {
         bucket_rename_blob,
     ],
     "storage.objects.insert": [
+        blob_upload_from_string_multipart,
+        blobwriter_write_multipart,
+        blob_create_resumable_upload_session,
+    ],
+    "storage.resumable.upload": [
         blob_upload_from_string,
         blob_upload_from_file,
         blob_upload_from_filename,
         blobwriter_write,
-        blob_create_resumable_upload_session,
     ],
     "storage.objects.patch": [
         blob_patch,
@@ -722,73 +821,6 @@ method_mapping = {
 }
 
 
-########################################################################################################################################
-### Pytest Fixtures to Populate Resources ##############################################################################################
-########################################################################################################################################
-
-
-@pytest.fixture
-def client():
-    client = storage.Client(
-        project=_CONF_TEST_PROJECT_ID,
-        credentials=AnonymousCredentials(),
-        client_options={"api_endpoint": _HOST},
-    )
-    return client
-
-
-@pytest.fixture
-def bucket(client):
-    bucket = client.bucket(uuid.uuid4().hex)
-    client.create_bucket(bucket)
-    yield bucket
-    try:
-        bucket.delete(force=True)
-    except NotFound:  # in cases where bucket is deleted within the test
-        pass
-
-
-@pytest.fixture
-def object(client, bucket):
-    blob = client.bucket(bucket.name).blob(uuid.uuid4().hex)
-    blob.upload_from_string(_STRING_CONTENT)
-    blob.reload()
-    yield blob
-    try:
-        blob.delete()
-    except NotFound:  # in cases where object is deleted within the test
-        pass
-
-
-@pytest.fixture
-def notification(client, bucket):
-    notification = client.bucket(bucket.name).notification(
-        topic_name=_CONF_TEST_PUBSUB_TOPIC_NAME
-    )
-    notification.create()
-    notification.reload()
-    yield notification
-    try:
-        notification.delete()
-    except NotFound:  # in cases where notification is deleted within the test
-        pass
-
-
-@pytest.fixture
-def hmac_key(client):
-    hmac_key, _secret = client.create_hmac_key(
-        service_account_email=_CONF_TEST_SERVICE_ACCOUNT_EMAIL,
-        project_id=_CONF_TEST_PROJECT_ID,
-    )
-    yield hmac_key
-    try:
-        hmac_key.state = "INACTIVE"
-        hmac_key.update()
-        hmac_key.delete()
-    except NotFound:  # in cases where hmac_key is deleted within the test
-        pass
-
-
 ########################################################################################################################################
 ### Helper Methods for Testbench Retry Test API ########################################################################################
 ########################################################################################################################################
@@ -833,7 +865,15 @@ def _get_retry_test(host, id):
 
 
 def _run_retry_test(
-    host, id, lib_func, _preconditions, bucket, object, notification, hmac_key
+    host,
+    id,
+    lib_func,
+    _preconditions,
+    bucket,
+    object,
+    notification,
+    hmac_key,
+    file_data,
 ):
     """
     To execute tests against the list of instrucions sent to the Retry Test API,
@@ -855,6 +895,7 @@ def _run_retry_test(
         object=object,
         notification=notification,
         hmac_key=hmac_key,
+        file_data=file_data,
     )
 
 
@@ -874,7 +915,16 @@ def _delete_retry_test(host, id):
 
 
 def run_test_case(
-    scenario_id, method, case, lib_func, host, bucket, object, notification, hmac_key
+    scenario_id,
+    method,
+    case,
+    lib_func,
+    host,
+    bucket,
+    object,
+    notification,
+    hmac_key,
+    file_data,
 ):
     scenario = _CONFORMANCE_TESTS[scenario_id - 1]
     expect_success = scenario["expectSuccess"]
@@ -901,6 +951,7 @@ def run_test_case(
             object,
             notification,
             hmac_key,
+            file_data,
         )
     except Exception as e:
         logging.exception(
@@ -948,11 +999,12 @@ with subprocess.Popen(_RUN_CMD) as proc:
         for i, c in enumerate(cases):
             for m in methods:
                 method_name = m["name"]
-                if method_name not in method_mapping:
+                method_group = m["group"] if m.get("group", None) else m["name"]
+                if method_group not in method_mapping:
                     logging.info("No tests for operation {}".format(method_name))
                     continue
 
-                for lib_func in method_mapping[method_name]:
+                for lib_func in method_mapping[method_group]:
                     test_name = "test-S{}-{}-{}-{}".format(
                         id, method_name, lib_func.__name__, i
                     )
