@@ -19,14 +19,6 @@ from google.cloud.storage import _helpers
 import mock
 
 
-try:
-    ConnectionError
-except NameError:
-    _HAS_STDLIB_CONNECTION_ERROR = False
-else:
-    _HAS_STDLIB_CONNECTION_ERROR = True
-
-
 class Test_should_retry(unittest.TestCase):
     def _call_fut(self, exc):
         from google.cloud.storage import retry
@@ -79,9 +71,6 @@ class Test_should_retry(unittest.TestCase):
         exc = ValueError("testing")
         self.assertFalse(self._call_fut(exc))
 
-    @unittest.skipUnless(
-        _HAS_STDLIB_CONNECTION_ERROR, "No builtin 'ConnectionError' in Python 2",
-    )
     def test_w_stdlib_connection_error(self):
         exc = ConnectionError()
         self.assertTrue(self._call_fut(exc))
